@@ -13,6 +13,8 @@
 #
 # Copyright Buildbot Team Members
 
+from __future__ import annotations
+
 import multiprocessing
 import os.path
 import socket
@@ -22,7 +24,6 @@ import time
 from twisted.application import service
 from twisted.internet import defer
 from twisted.internet import reactor
-from twisted.python import failure
 from twisted.python import log
 from twisted.spread import pb
 
@@ -161,14 +162,14 @@ class ProtocolCommandBase:
 
 
 class WorkerForBuilderBase(service.Service):
-    ProtocolCommand = ProtocolCommandBase
+    ProtocolCommand: type[ProtocolCommandBase] = ProtocolCommandBase
 
 
 class BotBase(service.MultiService):
     """I represent the worker-side bot."""
 
-    name = "bot"
-    WorkerForBuilder = WorkerForBuilderBase
+    name: str | None = "bot"  # type: ignore[assignment]
+    WorkerForBuilder: type[WorkerForBuilderBase] = WorkerForBuilderBase
 
     os_release_file = "/etc/os-release"
 
@@ -235,8 +236,8 @@ class BotBase(service.MultiService):
                     with open(filename) as fin:
                         try:
                             files[f] = bytes2unicode(fin.read())
-                        except UnicodeDecodeError:
-                            log.err(failure.Failure(), f'error while reading file: {filename}')
+                        except UnicodeDecodeError as e:
+                            log.err(e, f'error while reading file: {filename}')
 
         self._read_os_release(self.os_release_file, files)
 
@@ -245,8 +246,7 @@ class BotBase(service.MultiService):
                 self.numcpus = multiprocessing.cpu_count()
             except NotImplementedError:
                 log.msg(
-                    "warning: could not detect the number of CPUs for "
-                    "this worker. Assuming 1 CPU."
+                    "warning: could not detect the number of CPUs for this worker. Assuming 1 CPU."
                 )
                 self.numcpus = 1
         files['environ'] = os.environ.copy()
