@@ -42,9 +42,9 @@ class MakeDirectory(base.Command):
                 if not os.path.isdir(dirname):
                     os.makedirs(dirname)
             except OSError as e:
-                self.log_msg("MakeDirectory {0} failed: {1}".format(dirname, e))
+                self.log_msg(f"MakeDirectory {dirname} failed: {e}")
                 self.sendStatus([
-                    ('header', '{0}: {1}: {2}'.format(self.header, e.strerror, dirname)),
+                    ('header', f'{self.header}: {e.strerror}: {dirname}'),
                     ('rc', e.errno),
                 ])
                 return
@@ -124,14 +124,13 @@ class RemoveDirectory(base.Command):
         # permissions and re-try the rm -rf.
         if not chmodDone:
             rc = yield self._tryChmod(rc, path)
-        defer.returnValue(rc)
+        return rc
 
     @defer.inlineCallbacks
     def _tryChmod(self, rc, path):
         assert isinstance(rc, int)
         if rc == 0:
-            defer.returnValue(0)
-            return  # pragma: no cover
+            return 0
         # Attempt a recursive chmod and re-try the rm -rf after.
 
         command = ["chmod", "-Rf", "u+rwx", path]
@@ -157,7 +156,7 @@ class RemoveDirectory(base.Command):
         self.command = c
         rc = yield c.start()
         rc = yield self._clobber(rc, path, True)
-        defer.returnValue(rc)
+        return rc
 
 
 class CopyDirectory(base.Command):
@@ -194,15 +193,14 @@ class CopyDirectory(base.Command):
             @d.addCallback
             def send_rc(rc):
                 self.sendStatus([('rc', rc)])
+
         else:
             if not os.path.exists(os.path.dirname(to_path)):
                 os.makedirs(os.path.dirname(to_path))
             if os.path.exists(to_path):
                 # I don't think this happens, but just in case..
                 self.log_msg(
-                    ("cp target '{0}' already exists -- cp will not do what you think!").format(
-                        to_path
-                    )
+                    f"cp target '{to_path}' already exists -- cp will not do what you think!"
                 )
 
             if platform.system().lower().find('solaris') >= 0:
@@ -243,9 +241,9 @@ class StatFile(base.Command):
             stat = os.stat(filename)
             self.sendStatus([('stat', tuple(stat)), ('rc', 0)])
         except OSError as e:
-            self.log_msg("StatFile {0} failed: {1}".format(filename, e))
+            self.log_msg(f"StatFile {filename} failed: {e}")
             self.sendStatus([
-                ('header', '{0}: {1}: {2}'.format(self.header, e.strerror, filename)),
+                ('header', f'{self.header}: {e.strerror}: {filename}'),
                 ('rc', e.errno),
             ])
 
@@ -260,16 +258,12 @@ class GlobPath(base.Command):
         pathname = self.args['path']
 
         try:
-            # recursive matching is only support in python3.5+
-            if sys.version_info[:2] >= (3, 5):
-                files = glob.glob(pathname, recursive=True)
-            else:
-                files = glob.glob(pathname)
+            files = glob.glob(pathname, recursive=True)
             self.sendStatus([('files', files), ('rc', 0)])
         except OSError as e:
-            self.log_msg("GlobPath {0} failed: {1}".format(pathname, e))
+            self.log_msg(f"GlobPath {pathname} failed: {e}")
             self.sendStatus([
-                ('header', '{0}: {1}: {2}'.format(self.header, e.strerror, pathname)),
+                ('header', f'{self.header}: {e.strerror}: {pathname}'),
                 ('rc', e.errno),
             ])
 
@@ -287,9 +281,9 @@ class ListDir(base.Command):
             files = os.listdir(dirname)
             self.sendStatus([('files', files), ('rc', 0)])
         except OSError as e:
-            self.log_msg("ListDir {0} failed: {1}".format(dirname, e))
+            self.log_msg(f"ListDir {dirname} failed: {e}")
             self.sendStatus([
-                ('header', '{0}: {1}: {2}'.format(self.header, e.strerror, dirname)),
+                ('header', f'{self.header}: {e.strerror}: {dirname}'),
                 ('rc', e.errno),
             ])
 
@@ -307,8 +301,8 @@ class RemoveFile(base.Command):
             os.remove(pathname)
             self.sendStatus([('rc', 0)])
         except OSError as e:
-            self.log_msg("remove file {0} failed: {1}".format(pathname, e))
+            self.log_msg(f"remove file {pathname} failed: {e}")
             self.sendStatus([
-                ('header', '{0}: {1}: {2}'.format(self.header, e.strerror, pathname)),
+                ('header', f'{self.header}: {e.strerror}: {pathname}'),
                 ('rc', e.errno),
             ])
