@@ -17,9 +17,12 @@
 Support for running 'shell commands'
 """
 
+from __future__ import annotations
+
 import os
 import pprint
 import re
+import shlex
 import signal
 import stat
 import subprocess
@@ -27,13 +30,13 @@ import sys
 import traceback
 from codecs import getincrementaldecoder
 from tempfile import NamedTemporaryFile
+from typing import TYPE_CHECKING
 
 from twisted.internet import defer
 from twisted.internet import error
 from twisted.internet import protocol
 from twisted.internet import reactor
 from twisted.internet import task
-from twisted.python import failure
 from twisted.python import log
 from twisted.python import runtime
 from twisted.python.win32 import quoteArguments
@@ -44,8 +47,16 @@ from buildbot_worker.compat import bytes2unicode
 from buildbot_worker.compat import unicode2bytes
 from buildbot_worker.exceptions import AbandonChain
 
+if TYPE_CHECKING:
+    import datetime
+
 if runtime.platformType == 'posix':
     from twisted.internet.process import Process
+if runtime.platformType == 'win32':
+    import win32api
+    import win32con
+    import win32job
+    import win32process
 
 
 def win32_batch_quote(cmd_list, unicode_encoding='utf-8'):
@@ -65,13 +76,10 @@ def win32_batch_quote(cmd_list, unicode_encoding='utf-8'):
 
 def shell_quote(cmd_list, unicode_encoding='utf-8'):
     # attempt to quote cmd_list such that a shell will properly re-interpret
-    # it.  The pipes module is only available on UNIX; also, the quote
-    # function is undocumented (although it looks like it will be documented
-    # soon: http://bugs.python.org/issue9723). Finally, it has a nasty bug
-    # in some versions where an empty string is not quoted.
+    # it.  The shlex module is only designed for UNIX;
     #
     # So:
-    #  - use pipes.quote on UNIX, handling '' as a special case
+    #  - use shlex.quote on UNIX, handling '' as a special case
     #  - use our own custom function on Windows
     if isinstance(cmd_list, bytes):
         cmd_list = bytes2unicode(cmd_list, unicode_encoding)
@@ -79,19 +87,16 @@ def shell_quote(cmd_list, unicode_encoding='utf-8'):
     if runtime.platformType == 'win32':
         return win32_batch_quote(cmd_list, unicode_encoding)
 
-    # only available on unix
-    import pipes  # pylint: disable=import-outside-toplevel
-
     def quote(e):
         if not e:
             return '""'
         e = bytes2unicode(e, unicode_encoding)
-        return pipes.quote(e)
+        return shlex.quote(e)
 
-    return " ".join([quote(e) for e in cmd_list])
+    return " ".join(quote(e) for e in cmd_list)
 
 
-class LogFileWatcher(object):
+class LogFileWatcher:
     POLL_INTERVAL = 2
 
     def __init__(self, command, name, logfile, follow=False, poll=True):
@@ -101,7 +106,7 @@ class LogFileWatcher(object):
         decoderFactory = getincrementaldecoder(self.command.unicode_encoding)
         self.logDecode = decoderFactory(errors='replace')
 
-        self.command.log_msg("LogFileWatcher created to watch {0}".format(logfile))
+        self.command.log_msg(f"LogFileWatcher created to watch {logfile}")
         # we are created before the ShellCommand starts. If the logfile we're
         # supposed to be watching already exists, record its size and
         # ctime/mtime so we can tell when it starts to change.
@@ -206,7 +211,7 @@ class RunProcessPP(protocol.ProcessProtocol):
 
         if self.command.useProcGroup:
             if self.debug:
-                self.command.log_msg("pid {0} set as subprocess pgid".format(self.transport.pid))
+                self.command.log_msg(f"pid {self.transport.pid} set as subprocess pgid")
             self.transport.pgid = self.transport.pid
 
         if self.pending_stdin:
@@ -231,7 +236,7 @@ class RunProcessPP(protocol.ProcessProtocol):
 
     def processEnded(self, status_object):
         if self.debug:
-            self.command.log_msg("RunProcessPP.processEnded {0}".format(status_object))
+            self.command.log_msg(f"RunProcessPP.processEnded {status_object}")
         # status_object is a Failure wrapped around an
         # error.ProcessTerminated or and error.ProcessDone.
         # requires twisted >= 1.0.4 to overcome a bug in process.py
@@ -252,7 +257,7 @@ class RunProcessPP(protocol.ProcessProtocol):
         self.command.finished(sig, rc)
 
 
-class RunProcess(object):
+class RunProcess:
     """
     This is a helper class, used by worker commands to run programs in a child
     shell.
@@ -262,8 +267,8 @@ class RunProcess(object):
     interruptSignal = "KILL"
 
     # For sending elapsed time:
-    startTime = None
-    elapsedTime = None
+    startTime: datetime.datetime | None = None
+    elapsedTime: datetime.timedelta | None = None
 
     # For scheduling future events
     _reactor = reactor
@@ -286,6 +291,7 @@ class RunProcess(object):
         sendRC=True,
         timeout=None,
         maxTime=None,
+        max_lines=None,
         sigtermTime=None,
         initialStdin=None,
         keepStdout=False,
@@ -308,8 +314,6 @@ class RunProcess(object):
         @param useProcGroup: (default True) use a process group for non-PTY
             process invocations
         """
-        self.command_id = command
-
         if logfiles is None:
             logfiles = {}
 
@@ -321,6 +325,9 @@ class RunProcess(object):
                 return w
 
             command = [obfus(w) for w in command]
+
+        self.command_id = command
+
         # We need to take unicode commands and arguments and encode them using
         # the appropriate encoding for the worker.  This is mostly platform
         # specific, but can be overridden in the worker's buildbot.tac file.
@@ -352,6 +359,8 @@ class RunProcess(object):
         self.unicode_encoding = unicode_encoding
         self.send_update = send_update
         self.process = None
+        self.line_count = 0
+        self.max_line_kill = False
         if not os.path.exists(workdir):
             os.makedirs(workdir)
         if environ:
@@ -383,8 +392,7 @@ class RunProcess(object):
                 if v is not None:
                     if not isinstance(v, str):
                         raise RuntimeError(
-                            "'env' values must be strings or "
-                            "lists; key '{0}' is incorrect".format(key)
+                            f"'env' values must be strings or lists; key '{key}' is incorrect"
                         )
                     newenv[key] = p.sub(subst, v)
 
@@ -397,15 +405,17 @@ class RunProcess(object):
         self.ioTimeoutTimer = None
         self.sigtermTime = sigtermTime
         self.maxTime = maxTime
+        self.max_lines = max_lines
         self.maxTimeoutTimer = None
         self.killTimer = None
         self.keepStdout = keepStdout
         self.keepStderr = keepStderr
+        self.job_object = None
 
         assert usePTY in (
             True,
             False,
-        ), "Unexpected usePTY argument value: {!r}. Expected boolean.".format(usePTY)
+        ), f"Unexpected usePTY argument value: {usePTY!r}. Expected boolean."
         self.usePTY = usePTY
 
         # usePTY=True is a convenience for cleaning up all children and
@@ -440,10 +450,10 @@ class RunProcess(object):
             self.logFileWatchers.append(w)
 
     def log_msg(self, msg):
-        log.msg("(command {0}): {1}".format(self.command_id, msg))
+        log.msg(f"(command {self.command_id}): {msg}")
 
     def __repr__(self):
-        return "<{0} '{1}'>".format(self.__class__.__name__, self.fake_command)
+        return f"<{self.__class__.__name__} '{self.fake_command}'>"
 
     def start(self):
         # return a Deferred which fires (with the exit code) when the command
@@ -456,12 +466,12 @@ class RunProcess(object):
         try:
             self._startCommand()
         except Exception as e:
-            log.err(failure.Failure(), "error in RunProcess._startCommand")
-            self.send_update([('stderr', "error in RunProcess._startCommand (%s)\n" % str(e))])
+            log.err(e, "error in RunProcess._startCommand")
+            self.send_update([('stderr', f"error in RunProcess._startCommand ({e!s})\n")])
 
             self.send_update([('stderr', traceback.format_exc())])
             # pretend it was a shell error
-            self.deferred.errback(AbandonChain(-1, 'Got exception (%s)' % str(e)))
+            self.deferred.errback(AbandonChain(-1, f'Got exception ({e!s})'))
         return self.deferred
 
     def _startCommand(self):
@@ -523,28 +533,28 @@ class RunProcess(object):
         self.send_update([('header', display + "\n")])
 
         # then comes the secondary information
-        msg = " in dir {0}".format(self.workdir)
+        msg = f" in dir {self.workdir}"
         if self.timeout:
             if self.timeout == 1:
                 unit = "sec"
             else:
                 unit = "secs"
-            msg += " (timeout {0} {1})".format(self.timeout, unit)
+            msg += f" (timeout {self.timeout} {unit})"
         if self.maxTime:
             if self.maxTime == 1:
                 unit = "sec"
             else:
                 unit = "secs"
-            msg += " (maxTime {0} {1})".format(self.maxTime, unit)
+            msg += f" (maxTime {self.maxTime} {unit})"
         self.log_msg(" " + msg)
         self.send_update([('header', msg + "\n")])
 
-        msg = " watching logfiles {0}".format(self.logfiles)
+        msg = f" watching logfiles {self.logfiles}"
         self.log_msg(" " + msg)
         self.send_update([('header', msg + "\n")])
 
         # then the obfuscated command array for resolving unambiguity
-        msg = " argv: {0}".format(self.fake_command)
+        msg = f" argv: {self.fake_command}"
         self.log_msg(" " + msg)
         self.send_update([('header', msg + "\n")])
 
@@ -553,19 +563,16 @@ class RunProcess(object):
             msg = " environment:\n"
             env_names = sorted(self.environ.keys())
             for name in env_names:
-                msg += "  {0}={1}\n".format(
-                    bytes2unicode(name, encoding=self.unicode_encoding),
-                    bytes2unicode(self.environ[name], encoding=self.unicode_encoding),
-                )
-            self.log_msg(" environment:\n{0}".format(pprint.pformat(self.environ)))
+                msg += f"  {bytes2unicode(name, encoding=self.unicode_encoding)}={bytes2unicode(self.environ[name], encoding=self.unicode_encoding)}\n"
+            self.log_msg(f" environment:\n{pprint.pformat(self.environ)}")
             self.send_update([('header', msg)])
 
         if self.initialStdin:
-            msg = " writing {0} bytes to stdin".format(len(self.initialStdin))
+            msg = f" writing {len(self.initialStdin)} bytes to stdin"
             self.log_msg(" " + msg)
             self.send_update([('header', msg + "\n")])
 
-        msg = " using PTY: {0}".format(bool(self.usePTY))
+        msg = f" using PTY: {bool(self.usePTY)}"
         self.log_msg(" " + msg)
         self.send_update([('header', msg + "\n")])
 
@@ -593,6 +600,20 @@ class RunProcess(object):
         for w in self.logFileWatchers:
             w.start()
 
+    def _create_job_object(self):
+        job = win32job.CreateJobObject(None, "")
+        extented_info = win32job.QueryInformationJobObject(
+            job, win32job.JobObjectExtendedLimitInformation
+        )
+        extented_info['BasicLimitInformation']['LimitFlags'] = (
+            win32job.JOB_OBJECT_LIMIT_KILL_ON_JOB_CLOSE
+            | win32job.JOB_OBJECT_TERMINATE_AT_END_OF_JOB
+        )
+        win32job.SetInformationJobObject(
+            job, win32job.JobObjectExtendedLimitInformation, extented_info
+        )
+        return job
+
     def _spawnProcess(
         self,
         processProtocol,
@@ -610,16 +631,30 @@ class RunProcess(object):
         if env is None:
             env = {}
 
+        if runtime.platformType == 'win32':
+            if self.using_comspec:
+                process = self._spawnAsBatch(
+                    processProtocol, executable, args, env, path, usePTY=usePTY
+                )
+            else:
+                process = reactor.spawnProcess(
+                    processProtocol, executable, args, env, path, usePTY=usePTY
+                )
+            pHandle = win32api.OpenProcess(win32con.PROCESS_ALL_ACCESS, False, int(process.pid))
+            if win32process.GetExitCodeProcess(pHandle) == win32con.STILL_ACTIVE:
+                # use JobObject to group subprocesses
+                self.job_object = self._create_job_object()
+                win32job.AssignProcessToJobObject(self.job_object, pHandle)
+            return process
+
         # use the ProcGroupProcess class, if available
-        if runtime.platformType == 'posix':
+        elif runtime.platformType == 'posix':
             if self.useProcGroup and not usePTY:
                 return ProcGroupProcess(
                     reactor, executable, args, env, path, processProtocol, uid, gid, childFDs
                 )
 
         # fall back
-        if self.using_comspec:
-            return self._spawnAsBatch(processProtocol, executable, args, env, path, usePTY=usePTY)
         return reactor.spawnProcess(processProtocol, executable, args, env, path, usePTY=usePTY)
 
     def _spawnAsBatch(self, processProtocol, executable, args, env, path, usePTY):
@@ -653,6 +688,7 @@ class RunProcess(object):
 
     def addStdout(self, data):
         if self.sendStdout:
+            self._check_max_lines(data)
             self.send_update([('stdout', data)])
 
         if self.keepStdout:
@@ -662,6 +698,7 @@ class RunProcess(object):
 
     def addStderr(self, data):
         if self.sendStderr:
+            self._check_max_lines(data)
             self.send_update([('stderr', data)])
 
         if self.keepStderr:
@@ -689,43 +726,57 @@ class RunProcess(object):
             rc = -1
         if self.sendRC:
             if sig is not None:
-                self.send_update([('header', "process killed by signal {0}\n".format(sig))])
+                self.send_update([('header', f"process killed by signal {sig}\n")])
             self.send_update([('rc', rc)])
-        self.send_update([('header', "elapsedTime={0:0.6f}\n".format(self.elapsedTime))])
+        self.send_update([('header', f"elapsedTime={self.elapsedTime:0.6f}\n")])
         self._cancelTimers()
         d = self.deferred
         self.deferred = None
         if d:
             d.callback(rc)
         else:
-            self.log_msg("Hey, command {0} finished twice".format(self))
+            self.log_msg(f"Hey, command {self} finished twice")
 
     def failed(self, why):
-        self.log_msg("RunProcess.failed: command failed: {0}".format(why))
+        self.log_msg(f"RunProcess.failed: command failed: {why}")
         self._cancelTimers()
         d = self.deferred
         self.deferred = None
         if d:
             d.errback(why)
         else:
-            self.log_msg("Hey, command {0} finished twice".format(self))
+            self.log_msg(f"Hey, command {self} finished twice")
 
     def doTimeout(self):
         self.ioTimeoutTimer = None
-        msg = "command timed out: {0} seconds without output running {1}".format(
-            self.timeout, self.fake_command
+        msg = (
+            f"command timed out: {self.timeout} seconds without output running {self.fake_command}"
         )
         self.send_update([("failure_reason", "timeout_without_output")])
         self.kill(msg)
 
     def doMaxTimeout(self):
         self.maxTimeoutTimer = None
-        msg = "command timed out: {0} seconds elapsed running {1}".format(
-            self.maxTime, self.fake_command
-        )
+        msg = f"command timed out: {self.maxTime} seconds elapsed running {self.fake_command}"
         self.send_update([("failure_reason", "timeout")])
         self.kill(msg)
 
+    def _check_max_lines(self, data):
+        if self.max_lines is not None:
+            self.line_count += len(re.findall(r"\r\n|\r|\n", data))
+            if self.line_count > self.max_lines and not self.max_line_kill:
+                self.pp.transport.closeStdout()
+                self.max_line_kill = True
+                self.do_max_lines()
+
+    def do_max_lines(self):
+        msg = (
+            f"command exceeds max lines: {self.line_count}/{self.max_lines} "
+            f"written/allowed running {self.fake_command}"
+        )
+        self.send_update([("failure_reason", "max_lines_failure")])
+        self.kill(msg)
+
     def isDead(self):
         if self.process.pid is None:
             return True
@@ -753,6 +804,10 @@ class RunProcess(object):
             # blocks here until the process has terminated, while closing
             # stderr. This is weird.
             self.pp.transport.loseConnection()
+        elif runtime.platformType == 'win32':
+            if self.job_object is not None:
+                win32job.TerminateJobObject(self.job_object, 0)
+                self.job_object.Close()
 
         if self.deferred:
             # finished ought to be called momentarily. Just in case it doesn't,
@@ -766,42 +821,40 @@ class RunProcess(object):
             sig = getattr(signal, "SIG" + interruptSignal, None)
 
             if sig is None:
-                self.log_msg("signal module is missing SIG{0}".format(interruptSignal))
+                self.log_msg(f"signal module is missing SIG{interruptSignal}")
             elif not hasattr(os, "kill"):
                 self.log_msg("os module is missing the 'kill' function")
             elif self.process.pgid is None:
                 self.log_msg("self.process has no pgid")
             else:
-                self.log_msg("trying to kill process group {0}".format(self.process.pgid))
+                self.log_msg(f"trying to kill process group {self.process.pgid}")
                 try:
                     os.killpg(self.process.pgid, sig)
-                    self.log_msg(" signal {0} sent successfully".format(sig))
+                    self.log_msg(f" signal {sig} sent successfully")
                     self.process.pgid = None
                     hit = 1
                 except OSError:
-                    self.log_msg(
-                        'failed to kill process group (ignored): {0}'.format((sys.exc_info()[1]))
-                    )
+                    self.log_msg(f'failed to kill process group (ignored): {sys.exc_info()[1]}')
                     # probably no-such-process, maybe because there is no process
                     # group
 
         elif runtime.platformType == "win32":
             if interruptSignal is None:
                 self.log_msg("interruptSignal==None, only pretending to kill child")
-            elif self.process.pid is not None:
+            elif self.process.pid is not None or self.job_object is not None:
                 if interruptSignal == "TERM":
-                    self._taskkill(self.process.pid, force=False)
+                    self._win32_taskkill(self.process.pid, force=False)
                     hit = 1
                 elif interruptSignal == "KILL":
-                    self._taskkill(self.process.pid, force=True)
+                    self._win32_taskkill(self.process.pid, force=True)
                     hit = 1
 
         # try signalling the process itself (works on Windows too, sorta)
         if not hit:
             try:
-                self.log_msg("trying process.signalProcess('{0}')".format(interruptSignal))
+                self.log_msg(f"trying process.signalProcess('{interruptSignal}')")
                 self.process.signalProcess(interruptSignal)
-                self.log_msg(" signal {0} sent successfully".format(interruptSignal))
+                self.log_msg(f" signal {interruptSignal} sent successfully")
                 hit = 1
             except OSError:
                 log.err("from process.signalProcess:")
@@ -813,25 +866,34 @@ class RunProcess(object):
 
         return hit
 
-    def _taskkill(self, pid, force):
+    def _win32_taskkill(self, pid, force):
         try:
             if force:
-                cmd = "TASKKILL /F /PID {0} /T".format(pid)
+                cmd = f"TASKKILL /F /PID {pid} /T"
             else:
-                cmd = "TASKKILL /PID {0} /T".format(pid)
-
-            self.log_msg("using {0} to kill pid {1}".format(cmd, pid))
+                cmd = f"TASKKILL /PID {pid} /T"
+            if self.job_object is not None:
+                pr_info = win32job.QueryInformationJobObject(
+                    self.job_object, win32job.JobObjectBasicProcessIdList
+                )
+                if force or len(pr_info) < 2:
+                    win32job.TerminateJobObject(self.job_object, 1)
+            self.log_msg(f"terminating job object with pids {pr_info!s}")
+            if pid is None:
+                return
+            self.log_msg(f"using {cmd} to kill pid {pid}")
             subprocess.check_call(cmd)
-            self.log_msg("taskkill'd pid {0}".format(pid))
-
+            self.log_msg(f"taskkill'd pid {pid}")
+        except win32job.error:
+            self.log_msg("failed to terminate job object")
         except subprocess.CalledProcessError as e:
             # taskkill may return 128 or 255 as exit code when the child has already exited.
             # We can't handle this race condition in any other way than just interpreting the kill
             # action as successful
             if e.returncode in (128, 255):
-                self.log_msg("taskkill didn't find pid {0} to kill".format(pid))
+                self.log_msg(f"taskkill didn't find pid {pid} to kill")
             else:
-                self.log_msg("taskkill failed to kill process {0}: {1}".format(pid, e))
+                self.log_msg(f"taskkill failed to kill process {pid}: {e}")
 
     def kill(self, msg):
         # This may be called by the timeout, or when the user has decided to
