@@ -21,7 +21,6 @@
 import datetime
 import os
 import re
-import sys
 from subprocess import PIPE
 from subprocess import STDOUT
 from subprocess import Popen
@@ -36,7 +35,7 @@ def gitDescribeToPep440(version):
 
     VERSION_MATCH = re.compile(
         r'(?P<major>\d+)\.(?P<minor>\d+)\.(?P<patch>\d+)(\.post(?P<post>\d+))?(-(?P<dev>\d+))?(-g(?P<commit>.+))?'
-    )  # noqa pylint: disable=line-too-long
+    )
     v = VERSION_MATCH.search(version)
     if v:
         major = int(v.group('major'))
@@ -45,10 +44,10 @@ def gitDescribeToPep440(version):
         if v.group('dev'):
             patch += 1
             dev = int(v.group('dev'))
-            return "{0}.{1}.{2}.dev{3}".format(major, minor, patch, dev)
+            return f"{major}.{minor}.{patch}.dev{dev}"
         if v.group('post'):
-            return "{0}.{1}.{2}.post{3}".format(major, minor, patch, v.group('post'))
-        return "{0}.{1}.{2}".format(major, minor, patch)
+            return "{}.{}.{}.post{}".format(major, minor, patch, v.group('post'))
+        return f"{major}.{minor}.{patch}"
 
     return v
 
@@ -60,21 +59,19 @@ def mTimeVersion(init_file):
         for f in files:
             m = max(os.path.getmtime(os.path.join(root, f)), m)
 
-    if sys.version_info >= (3, 3):
-        d = datetime.datetime.fromtimestamp(m, datetime.timezone.utc)
-    else:
-        d = datetime.datetime.utcfromtimestamp(m)
+    d = datetime.datetime.fromtimestamp(m, datetime.timezone.utc)
     return d.strftime("%Y.%m.%d")
 
 
-def getVersionFromArchiveId(git_archive_id='$Format:%ct %d$'):
+def getVersionFromArchiveId(git_archive_id='$Format:%ct %(describe:abbrev=10)$'):
     """Extract the tag if a source is from git archive.
 
     When source is exported via `git archive`, the git_archive_id init value is modified
     and placeholders are expanded to the "archived" revision:
 
         %ct: committer date, UNIX timestamp
-        %d: ref names, like the --decorate option of git-log
+        %(describe:abbrev=10): git-describe output, always abbreviating to 10 characters of commit ID.
+                               e.g. v3.10.0-850-g5bf957f89
 
     See man gitattributes(5) and git-log(1) (PRETTY FORMATS) for more details.
     """
@@ -83,17 +80,13 @@ def getVersionFromArchiveId(git_archive_id='$Format:%ct %d$'):
         # source was modified by git archive, try to parse the version from
         # the value of git_archive_id
 
-        match = re.search(r'tag:\s*v([^,)]+)', git_archive_id)
-        if match:
+        tstamp, _, describe_output = git_archive_id.strip().partition(' ')
+        if describe_output:
             # archived revision is tagged, use the tag
-            return gitDescribeToPep440(match.group(1))
+            return gitDescribeToPep440(describe_output)
 
         # archived revision is not tagged, use the commit date
-        tstamp = git_archive_id.strip().split()[0]
-        if sys.version_info >= (3, 3):
-            d = datetime.datetime.fromtimestamp(int(tstamp), datetime.timezone.utc)
-        else:
-            d = datetime.datetime.utcfromtimestamp(int(tstamp))
+        d = datetime.datetime.fromtimestamp(int(tstamp), datetime.timezone.utc)
         return d.strftime('%Y.%m.%d')
     return None
 
@@ -114,7 +107,7 @@ def getVersion(init_file):
         fn = os.path.join(cwd, 'VERSION')
         with open(fn) as f:
             return f.read().strip()
-    except IOError:
+    except OSError:
         pass
 
     version = getVersionFromArchiveId()
